local modName = g_currentModName;
local modDir = g_currentModDirectory;

Placeable = {};

Placeable.placeables = {};
function Placeable.addPlaceable(placeable)
	local xmlFilename = placeable.configFileName:lower();
	local list = Placeable.placeables[xmlFilename];
	if list == nil then
		list = {};
		Placeable.placeables[xmlFilename] = list;
	end;
	list[placeable] = placeable;
end;
function Placeable.removePlaceable(placeable)
	local xmlFilename = placeable.configFileName:lower();
	local list = Placeable.placeables[xmlFilename];
	if list ~= nil then
		list[placeable] = nil;
	end;
end;
function Placeable.getNumPlaceables(xmlFilename)
	local num = 0;
	local xmlFilename = xmlFilename:lower();
	--[[local list = Placeable.placeables[xmlFilename];
	if list ~= nil then
		for k in pairs(list) do
			num = num+1;
		end;
	end;]]
	-- improved version
	-- inserted by MaxT35, 28/04/16
	-- the same as getShowSellButton
	for n, v in pairs(g_currentMission.vehicles) do
		if v.configFileName:lower() == xmlFilename then
			num = num + 1;
		end;
	end;
	
	print(string.format("counting placeables: %s : count %d", xmlFilename, num));
	return num;
end;
function Placeable.getShowSellButton(xmlFilename)
	-- inserted by MaxT35, 28/04/16
	local xmlFilename = xmlFilename:lower();
	for n, v in pairs(g_currentMission.vehicles) do
		if v.configFileName:lower() == xmlFilename then
			return true;
		end;
	end;
	return false;
end;

function Placeable.prerequisitesPresent(specializations)
	return true;
end;

function Placeable:load(xmlFile)

	self.placementSizeX = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.placement#sizeX"), 1);
	self.placementSizeZ = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.placement#sizeZ"), 1);
	self.placementTestSizeX = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.placement#testSizeX"), 1);
	self.placementTestSizeZ = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.placement#testSizeZ"), 1);
	self.useRandomYRotation = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.placement#useRandomYRotation"), false);
	self.useManualYRotation = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.placement#useManualYRotation"), false);

	self.isPlaceableNonTabbable = true;

	for _, component in pairs(self.components) do
		g_currentMission:addNodeObject(component.node, self);
	end;
end;

function Placeable:delete()
	Placeable.removePlaceable(self);
	for _, component in pairs(self.components) do
		g_currentMission:removeNodeObject(component.node);
	end;
end;

function Placeable:readStream(streamId, connection)
	if connection:getIsServer() then

		for _, component in pairs(self.components) do
			g_currentMission:addNodeObject(component.node, self);
		end;
		Placeable.addPlaceable(self);
	end;
end;

function Placeable:writeStream(streamId, connection)
end;

function Placeable:readUpdateStream(streamId, timestamp, connection)
end;

function Placeable:writeUpdateStream(streamId, connection, dirtyMask)
end;

function Placeable:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Placeable:keyEvent(unicode, sym, modifier, isDown)
end;

function Placeable:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)

	local numMovingTools = getXMLInt(xmlFile, key.."#numMovingTools");
	if self.movingTools ~= nil and numMovingTools ~= nil then
	numMovingTools = math.min(numMovingTools, table.getn(self.movingTools));
	for i=1, numMovingTools do
		local tool = self.movingTools[i];
		local toolKey = key..string.format(".movingTool%d", i);
		local changed = false;
		if tool.transSpeed ~= nil then
			local newTrans = getXMLFloat(xmlFile, toolKey.."#curTrans");
			if newTrans ~= nil and math.abs(newTrans - tool.curTrans[tool.translationAxis]) > 0.0001 then
				tool.curTrans[tool.translationAxis] = newTrans;
				setTranslation(tool.node, unpack(tool.curTrans));
				changed = true;
			end;
		end;
		if tool.rotSpeed ~= nil then
			local newRot = getXMLFloat(xmlFile, toolKey.."#curRot");
			if newRot ~= nil and math.abs(newRot - tool.curRot[tool.rotationAxis]) > 0.0001 then
				tool.curRot[tool.rotationAxis] = newRot;
				setRotation(tool.node, unpack(tool.curRot));
				changed = true;
				end;
			end;
			if changed then
				Cylindered.setDirty(self, tool);
			end;
		end;
	end;
	return BaseMission.VEHICLE_LOAD_OK;
end

function Placeable:getSaveAttributesAndNodes(nodeIdent)

	local attributes = "";
	local nodes = "";
	if self.movingTools ~= nil then
		local numMovingTools = table.getn(self.movingTools);
		attributes = 'numMovingTools="'..numMovingTools..'"';
		for i=1, numMovingTools do
			local tool = self.movingTools[i];

			if i>1 then
				nodes = nodes.."\n";
			end;
			nodes = nodes..nodeIdent..'<movingTool'..string.format("%d", i);
			if tool.transSpeed ~= nil then
				nodes = nodes..' curTrans="'..tool.curTrans[tool.translationAxis]..'"';
			end;
			if tool.rotSpeed ~= nil then
				nodes = nodes..' curRot="'..tool.curRot[tool.rotationAxis]..'"';
			end;
			nodes = nodes..' />';
		end;
	end;
	return attributes,nodes;
end

function Placeable:update(dt)
end;

function Placeable:updateTick(dt)
end;

function Placeable:draw()
end;